/*******************************************************************************
Copyright 2020 Anders Gustafsson, anders.gustafsson@pedago.fi


*******************************************************************************/

/** INCLUDES *******************************************************/

#include "system.h"         // Defines config bits, FCY, VERSION etc

#include <libpic30.h>        // __delayXXX() functions macros defined here
#include <stdio.h>
#include <string.h>
#include "hil_kbd.h"
#include "ps2_kbd.h"
#include "ps2_mouse.h"
#include "vnc2.h"
#include "spi_slave.h"
/****************************************************************************
  Function:
    int main(void)

  Summary:
    main function

  Description:
    main function

  Precondition:
    None

  Parameters:
    None

  Return Values:
    int - exit code for main function

  Remarks:
 Note that we have FIFIs galore here. Reception for HIL and PS/2 is interrupt-driven
 * and data is put in FIFOs. In additon, data to be transmitted over HIL is
 * put in a FIFO. This is mainly to handle USB as a shift-keypress should result
 * in two separate scancodes being sent.
  ***************************************************************************/
//
int x=0;
int y=0;
int z=0;
int addr = 0;
int cmd = 0;
int data = 0;
int pollno = 0;
int Tempkey;
int Lastcode = 0;
int Laststat = 0;
int Lastbutton = 0;
int Outkey;
int Code2 = 0;          // Whether to take code 2, ie our shifted stuff
int LastWasExtended = 0;
//
char LastKey;
char mousebytes[3];
char ch;
int ptr = 0;
int mousex,mousey;
int usb = 0;

#define Release LATBbits.LATB15     
#define Extended LATBbits.LATB14

extern char HILAddress;
extern int PS2MouseRXCounter;

int HILTasks(void);
int PS2KeyboardTasks(void);
int PS2MouseTasks(void);

void HardReset(void);

int main(void)
{   
    //
    // Set 8MHz clock FRC. PLLDIV=2, gives 4MHz. Fed to PLL gives 96MHz
    // divided by 3 gives 32MHz cpu clock
    //
    CLKDIVbits.CPDIV = 0;   // Divide by nothing = 32MHz
    CLKDIVbits.RCDIV = 0;   // FRC Postscaler Select bits, divide by 0
                            // ie FRCDIV is 8MHz, same as FRC
                            // PLLDIV is set in config bits to 2 (system.c)
    TRISCbits.TRISC3 = 0;   // Mouse power
    //
    //
    //
    spi1_setup();
    //
    HILInit();

    PS2Init();
    PS2MouseInit();
    //
    while(1)
    {
        HILTasks();
        //
        while(spi1_test())
            USBTasks();
        PS2KeyboardTasks();
        PS2MouseTasks();

    }//end while
}//end main
//
// Check for byte on HIL and act on it
//
int HILTasks(void)
{
    if(!RX_test())
        return 0;
    //
    x = RX_getc();
    //HILPutch(x);
    y = ((x&0b0000111111111111)) ;       // Remove start bit, parity, stop
    addr = y & 0b111;
    addr = ReverseAddress(addr);
    cmd = (y & 0b1000)>>3;
    data = (y & 0b111111110000)>>4;
    z = ReverseByte(data);
    //if(z != 0x10) printf("\r\nadd:%X cmd:%X dta:%X %s  %d",addr,cmd,z, HILNames[z], HILAddress);
    //if(z == 0x10)
        //printf("\r%X -> %X add:%X cmd:%X dta:%X %s %d ", x, y,addr,cmd,z, HILNames[z], pollno++);
        //++pollno;
    //else
        //printf("\r\n%X -> %X add:%X cmd:%X dta:%X %s  ", x, y,addr,cmd,z, HILNames[z]);
        //++pollno;
    //if( z == 0x01 || z==0x02) printf("\r\n");
    HILParse(cmd, addr, z);
    return 1;
}

int PS2KeyboardTasks(void)
{
    //
    if(!PS2Test())
        return 0;
    //
    Tempkey=PS2getc();
    Release = 0;
    Extended = 0;
    Lastcode = 0;
    //printf("\r\nPS2key %X", Tempkey);
    //return 0;
    

    if(Tempkey == 0xE1) // This is Pause/Break
                        // it sends E012E07C
    {
        printf("\r\n Pause/break");
        //Extended = 1;
        Lastcode = 0;
        Tempkey=0xE114;
        Key = 0x9C;             // This is HIL Stop
        // consume seven scans
        PS2getc();
        PS2getc();
        PS2getc();
        PS2getc();
        PS2getc();
        PS2getc();
        PS2getc();             
    }
    //
    // This is the escape-sequence for most of the keys they could not fit
    // ie the numpad and stuff.
    // For example does numpad enter send
    // E05A on press
    // E0F05A on release
    //
    // We simply make the translation table input colum an integer
    // to lookup these
    //
    // When you hold dhow shift and press arrow up for example, then
    // the keyboard sends:
    // 12
    // E0 F0 12
    // E0 75
    // E0 F0 75
    // F0 12
    //
    // I have no idea what purpose that extra extended release has, but
    // it causes problems so I added a flag that mutes it unless the last
    // key was indeed shifted.
    //
    if(Tempkey == 0xE0)
    {
        //printf("\r\n extended E0");
        Extended = 1;
        Lastcode = Tempkey * 0x100;
        Tempkey=PS2getc();
        //printf("  %X ", Tempkey);
        if(Tempkey == 0x14)             // Right control is our pseudo-shift
        {
            Code2 = 1;
            printf(" code2 on");
        }

    }
    if(Tempkey == 0xF0)                 // This is sent for release
    {                                   // Ie 1C is "A" and F01C is when you
        Release = 1;                    // release the A-key
        Tempkey=PS2getc();
        if(Extended)
        {
            //printf("\r\n Extended Release %X %d", Tempkey,LastWasExtended);
            // Mute the extended release unless the last key was indeed
            // extended
            if(LastWasExtended == 0) Tempkey = 0;
            //Lastcode = 0;
            if(Tempkey == 0x14)             // Right control is released
            {
                Code2 = 0;
                printf(" code2 off");
                return 1;
            }
        }
        //else printf("\r\n Release %X", Tempkey);
    }
    //printf("\r\n Tempkey: %X, Lastcode: %X ", Tempkey, Lastcode);
    if(Lastcode != 0) Tempkey = Tempkey + Lastcode; //0xE000; Unsigned stupid!
    //
    if(Extended == 1) LastWasExtended = 1;
    else LastWasExtended = 0;
    // ---------------------------------------------------------------
    // Note that this lookup routine is not terribly efficient
    //
    uint8_t index2;
    for(index2 = 0; index2 < sizeof(PS2keyTranslationTable)/sizeof(PS2_HP_TRANSLATION_TABLE_ENTRY); index2++)
    {
        //HILTasks();
        if(PS2keyTranslationTable[index2].key == Tempkey)
        {
            if(Code2)
                Outkey = PS2keyTranslationTable[index2].scancode2;
            else
                Outkey = PS2keyTranslationTable[index2].scancode;
            
            if (Release) Outkey++;
            //Key = Outkey&0xFF;
            HILPutTX(Outkey&0xFF);
            printf("\r\nP: %4X C: %2X ", Tempkey, Outkey&0xFF);
        }
    }
    // ---------------------------------------------------------------
    return 1;
}

int PS2MouseTasks(void)
{
     if(!PS2MouseTest())
         return 0;
    // Assemble mouse packets, three bytes
    //
    ch = PS2Mousegetc();
    //printf("\r\nptr:%d %X", ptr,ch);
    mousebytes[ptr++]=ch;
    //
    if(ptr>2)
    {
        printf("\r\nM %02X %02X %02X  ", mousebytes[0]&0xff,mousebytes[1]&0xff,mousebytes[2]&0xff);
        mousex = mousebytes[1];
        mousey = mousebytes[2];
        MouseStatus.status = mousebytes[0];
        if(!MouseStatus.one)
        {
            printf(" - Invalid status -");
            ++MouseErrors;
            if(MouseErrors > 0)
            {
                PS2MouseReset();
                MouseErrors = 0;
            }
        }
        printf("  %3d  %3d", mousex, mousey);
        ptr = 0;
        if(MouseStatus.one) Mouse = 1;  // Act only on valid packets!
    }
    return 1;
}
// Called on HIL Hard Reset
//
void HardReset(void)
{
    PS2MouseReset();
    PS2Reset();
}