//
// Includefile for hpkbd PS/2 keyboard handling
//
extern volatile unsigned int Counter;
extern volatile unsigned char Key;
extern int Code2;

typedef struct
{
    unsigned int key;
    char scancode;                  // HP scandode to send for dn. Up is the same code +1
    char scancode2;                 // Scancode to send if we hold down right Ctrl
} PS2_HP_TRANSLATION_TABLE_ENTRY;

#define PS2_NORMAL 0
#define PS2_EXTENDED 1
#define PS2_RELEASE 2
#define PS2_PRTSC 3
//
// Translate PS2 scancodes to HP HIL scancodes. Note that the HIL code is the
// down code, HIL release codes are all down+1
//
// Lookup routine is n0t very efficient, but can be improved by putting
// commonly used keys at the beginning.
//
static const PS2_HP_TRANSLATION_TABLE_ENTRY PS2keyTranslationTable[] = 
{
{ 0x0E, 0x7E ,0x00 },    // `
{ 0x16, 0x7C ,0x00 },    // 1
{ 0x1E, 0x7A ,0x00 },    // 2
{ 0x26, 0x78 ,0x00 },    // 3
{ 0x25, 0x76 ,0x00 },    // 4
{ 0x2E, 0x74 ,0x00 },    // 5
{ 0x36, 0x72 ,0x00 },    // 6
{ 0x3D, 0x70 ,0x00 },    // 7
{ 0x3E, 0xB0 ,0x00 },    // 8
{ 0x46, 0xB2 ,0x00 },    // 9
{ 0x45, 0xB4 ,0x00 },    // 0
{ 0x4E, 0xB6 ,0x00 },    // -
{ 0x55, 0xB8 ,0x00 },    // =
{ 0x66, 0xBA ,0x00 },    // Backspace
{ 0x0D, 0x6E ,0x00 },    // Tab
{ 0x15, 0x6C ,0x00 },    // q
{ 0x1D, 0x6A ,0x00 },    // w
{ 0x24, 0x68 ,0x00 },    // e
{ 0x2D, 0x66 ,0x00 },    // r
{ 0x2C, 0x64 ,0x00 },    // t
{ 0x35, 0x62 ,0x00 },    // y
{ 0x3C, 0x60 ,0x00 },    // u
{ 0x43, 0xC0 ,0x00 },    // i
{ 0x44, 0xC2 ,0x00 },    // o
{ 0x4D, 0xC4 ,0x00 },    // p
{ 0x54, 0xC6 ,0x00 },    // [
{ 0x5B, 0xC8 ,0x00 },    // ]
{ 0x58, 0x5E ,0x00 },    // Caps Lock
{ 0x1C, 0x5A ,0x38 },    // a
{ 0x1B, 0x58 ,0x00 },    // s
{ 0x23, 0x56 ,0x00 },    // d
{ 0x2B, 0x54 ,0x00 },    // f
{ 0x34, 0x52 ,0x00 },    // g
{ 0x33, 0x50 ,0x00 },    // h
{ 0x3B, 0xD0 ,0x00 },    // j
{ 0x42, 0xD2 ,0x00 },    // k
{ 0x4B, 0xD4 ,0x00 },    // l
{ 0x4C, 0xD6 ,0x00 },    // ;
{ 0x52, 0xD8 ,0x00 },    // '
//{ 0x5A, 0x9E ,0x00 },    // Enter
{ 0x5A, 0xDA ,0x00 },    //Enter = Return
{ 0x12, 0x0A ,0x00 },    //Left shift
{ 0x11, 0x07 ,0x00 },    //Left extend
{ 0x1A, 0x38 ,0x00 },    // z
{ 0x22, 0x36 ,0x00 },    // x
{ 0x21, 0x34 ,0x00 },    // c
{ 0x2A, 0x32 ,0x00 },    // v
{ 0x32, 0x30 ,0x00 },    // b
{ 0x31, 0xF0 ,0x00 },    // n
{ 0x3A, 0xE0 ,0x00 },    // m
{ 0x41, 0xE2 ,0x00 },    // ,
{ 0x49, 0xE4 ,0x00 },    // .
{ 0x4A, 0xE6 ,0x00 },    // /
{ 0x59, 0x08 ,0x00 },    //Right Shift
{ 0x14, 0x0C ,0x00 },    // Left Ctrl
{ 0x29, 0xF2 ,0x00 },    // Spacebar
{ 0x14, 0x0C ,0x00 },    // Right Ctrl
{ 0x67, 0xCC ,0x00 },    // Insert
{ 0x64, 0xCE ,0x00 },    // Delete
{ 0x6B, 0xF8 ,0x00 },    // Left Arrow
{ 0x75, 0xFC ,0x00 },    // Up Arrow
{ 0x72, 0xFA ,0x00 },    // Down Arrow
{ 0x74, 0xFE ,0x00 },    // Right Arrow
{ 0x6C, 0xDC ,0x00 },    //Home
{ 0x6B, 0x10 ,0x00 },    // Keypad 4
{ 0x69, 0x20 ,0x00 },    // Keypad 1
{ 0x77, 0xA0 ,0x00 },    //NUmlock = System
{ 0x75, 0x12 ,0x00 },    // Keypad 8
{ 0x73, 0x14 ,0x00 },    // Keypad 5
{ 0x72, 0x24 ,0x00 },    // Keypad 2
{ 0x70, 0x2C ,0x00 },    //Keypad 0
{ 0x7E, 0x0C ,0x00 },    //Stop (Scroll lock))
{ 0x7D, 0x16 ,0x00 },    // Keypad 9
{ 0x74, 0x18 ,0x00 },    // Keypad 6
{ 0x7A, 0x28 ,0x00 },    // Keypad 3
{ 0x71, 0xCE ,0x00 },    //Delete
{ 0x84, 0x2E ,0x00 },    // Keypad -
{ 0x79, 0x1E ,0x00 },    // Keypad Enter
{ 0x76, 0x3E ,0x00 },    //Esc
{ 0x05, 0x98 ,0x00 },    // F1
{ 0x06, 0x96 ,0x00 },    // F2
{ 0x04, 0x94 ,0x00 },    // F3
{ 0x0c, 0x92 ,0x00 },    // F4
{ 0x03, 0xA2 ,0x00 },    // F5
{ 0x0b, 0xA4 ,0x00 },    // F6
{ 0x83, 0xA6 ,0x00 },    // F7
{ 0x0A, 0xA8 ,0x00 },    // F8
{ 0xE1, 0x9C ,0x00 },    // Pause Break
{ 0x01, 0x4A ,0x9E },    // F9
{ 0x09, 0x42 ,0xAE },    // F10
{ 0x78, 0x46 ,0xAC },    // F11
{ 0x07, 0x4e ,0x00 },    // F12
{ 0x5D, 0xCA ,0x00 },    // */' right of -key
{ 0x61, 0xCA ,0x00 },    // < > | key r p annat stlle...
//
// Extended keys, prefixed with E0 or E1
//
{ 0xE06B, 0xF8 ,0x00 },    //Left arrow
{ 0xE074, 0xFE ,0x00 },    //Right arrow
{ 0xE075, 0xFC ,0x00 },    //Up arrow
{ 0xE072, 0xFA ,0x00 },    //Dn arrow
{ 0xE070, 0xcc ,0xBC },    //Ins char / Ins line
{ 0xE071, 0xce ,0xBE },    //del char / Del line
{ 0xE06C, 0xDC ,0x00 },    //Home
{ 0xE069, 0xEA ,0x00 },    //End = Select
{ 0xE07D, 0xDE ,0x00 },    //PgUp = Prev
{ 0xE07A, 0xEE ,0x00 },    //PgDn = Next
{ 0xE05A, 0xDA ,0x00 },    //Enter = Return
{ 0xE114, 0x0E ,0x00 },    //Pause/Brk = Break/Reset
{ 0xE07C, 0x90 ,0x00 },    //Prt Sc = Menu

};

//
// this is just to read hex values for config
//
//static const PS2_HP_TRANSLATION_TABLE_ENTRY PS2keyTranslationTable2[] = 
//{
//{ 0x16, 0x01 ,0x00 },    // 1
//{ 0x1E, 0x02 ,0x00 },    // 2
//{ 0x26, 0x03 ,0x00 },    // 3
//{ 0x25, 0x04 ,0x00 },    // 4
//{ 0x2E, 0x05 ,0x00 },    // 5
//{ 0x36, 0x06 ,0x00 },    // 6
//{ 0x3D, 0x07 ,0x00 },    // 7
//{ 0x3E, 0x08 ,0x00 },    // 8
//{ 0x46, 0x09 ,0x00 },    // 9
//{ 0x45, 0x00 ,0x00 },    // 0
//{ 0x1C, 0x0a ,0x00 },    // a
//{ 0x32, 0x0b ,0x00 },    // b
//{ 0x21, 0x0c ,0x00 },    // c
//{ 0x23, 0x0d ,0x00 },    // d
//{ 0x24, 0x0e ,0x00 },    // e
//{ 0x2B, 0x0f ,0x00 },    // f
//{ 0x5A, 0x20 ,0x00 },    //Enter = Return
//};

void PS2Init(void);
int PS2Test(void);
int PS2getc(void);
char PS2Scan2Hex (char);
void PS2Reset(void);

