// PIC24FJ64GB004 Family
//
#include <xc.h>			/* Device include file */
#include "spi.h"
void init_spi (char speed)		/* Initialize SPI port */
{
    // Set pins for SPI
    //
    TRISBbits.TRISB8 = 1;       // SPI In
    TRISBbits.TRISB9 = 0;       // SPI Out
    TRISCbits.TRISC4 = 0;       // SPI Clock
    //
    // Attach SPI1 module to I/O pads
	RPINR20 = 0x0008;           // SDI1 <-- RP8
	RPOR10 = 0x0008;             // SCK1OUT --> RP20
	RPOR4 = 0x0700;            // SDO1 --> RP9 

    if (speed)
    {
        // Set-up SPI bus, fast
        SPI1STATbits.SPIEN = 0;     // Disable SPI
        SPI1CON2bits.FRMEN = 0;
        SPI1CON2bits.SPIFSD = 0;
        SPI1CON1bits.DISSDO = 0;    // SDOx pin is controlled by the module
        SPI1CON1bits.MODE16 = 1;    // 1 = Communication is word-wide (16 bits)
        SPI1CON1bits.SMP = 1;       // Sample: 1 = Input data sampled at end of data output time
                                // 0 = Input data sampled at middle of data output time
        SPI1CON1bits.CKE = 1;       // Closk edge: 1 = Serial output data changes on transition from active clock state to Idle clock state (see bit 6)
                                // 0 = Serial output data changes on transition from Idle clock state to active clock state
        SPI1CON1bits.SSEN = 0;      // Slave Select not used
        SPI1CON1bits.CKP = 0;       // Clock polarity Idle state for clock is a low level; active state is a high level
        SPI1CON1bits.MSTEN = 1;     // Master mode enable bit
        SPI1CON1bits.SPRE = 7;      // Secondary prescale: 000 = 8:1, 111 = 1:1
        SPI1CON1bits.PPRE = 3;      // Primary prescale: 00 = 64:1, 11 = 1:1
        SPI1STATbits.SPIEN = 1;     // Enable SPI
    }
    else
    {
        // Set-up SPI bus, slow
        SPI1STATbits.SPIEN = 0;     // Disable SPI
        SPI1CON2bits.FRMEN = 0;
        SPI1CON2bits.SPIFSD = 0;
        SPI1CON1bits.DISSDO = 0;    // SDOx pin is controlled by the module
        SPI1CON1bits.MODE16 = 1;    // 1 = Communication is word-wide (16 bits)
        SPI1CON1bits.SMP = 1;       // Sample: 1 = Input data sampled at end of data output time
                                // 0 = Input data sampled at middle of data output time
        SPI1CON1bits.CKE = 1;       // Closk edge: 1 = Serial output data changes on transition from active clock state to Idle clock state (see bit 6)
                                // 0 = Serial output data changes on transition from Idle clock state to active clock state
        SPI1CON1bits.SSEN = 0;      // Slave Select not used
        SPI1CON1bits.CKP = 0;       // Clock polarity Idle state for clock is a low level; active state is a high level
        SPI1CON1bits.MSTEN = 1;     // Master mode enable bit
        SPI1CON1bits.SPRE = 0;      // Secondary prescale: 000 = 8:1, 111 = 1:1
        SPI1CON1bits.PPRE = 2;      // Primary prescale: 00 = 64:1, 11 = 1:1
        SPI1STATbits.SPIEN = 1;     // Enable SPI
    }

}


void xmit_spi (int d)		/* Send a byte over SPI */
{
    SPI1BUF = d;  //Send data
    while(SPI1STATbits.SPITBF);  //While full wait
    while(!SPI1STATbits.SPITBF);  //While full wait
    d = SPI1BUF;
}

int rcv_spi (void)			/* Send a 0xFF to the MMC and get the received byte */
{
    SPI1BUF = 0x00FF;
    while(SPI1STATbits.SPIRBF);  //While full wait
    while(!SPI1STATbits.SPIRBF);  //While full wait
    return (SPI1BUF);
}



