#include <xc.h>
#include "spi_slave.h"


#define BUFFER_SIZE 128

static volatile struct {
	int		ri, wi, ct;			/* Read index, Write index, Data counter */
	char	buff[BUFFER_SIZE];	/* FIFO buffer */
} SPI1RxFifo;
//
//
// Set up SPI1 as slave
void spi1_setup (void)
{
    // Set pins for SPI1
    //
    TRISBbits.TRISB8 = 1;       // SPI In
    TRISBbits.TRISB9 = 0;       // SPI Out
    TRISCbits.TRISC4 = 1;       // SPI Clock, input for slave
    TRISAbits.TRISA8 = 1;       // SPI CS
    //
    // Attach SPI1 module to I/O pads
	RPINR20bits.SDI1R = 0x0008; // SDI1 <-- RP8
    RPINR20bits.SCK1R = 0x14;   //SCK1IN <-- RP20
	//RPOR10 = 0x0008;          // SCK1OUT --> RP20
	RPOR4 = 0x0700;             // SDO1 --> RP9 
    //
    SPI1RxFifo.ct = 0;
    SPI1RxFifo.ri = 0;
    SPI1RxFifo.wi = 0;
    SPI1CON2bits.FRMEN = 0;         // No framed SPI
    SPI1CON2bits.SPIFSD = 1;        // Frame sync intput, slave
    SPI1CON1bits.DISSDO = 0;        // SDO2 used
    //_MODE16 = 1;
    SPI1CON1bits.MODE16 = 0;        // 1 = 16-bit mode 
    SPI1CON1bits.SMP = 1;           // Data sampled at end
    SPI1CON1bits.CKE = 0;           // Data changes from idle to active
    SPI1CON1bits.SSEN = 0;          //  SS2 not used
    SPI1CON1bits.CKP = 0;           // Clock polarity = idle low
    SPI1CON1bits.MSTEN = 0;         // Slave mode
    SPI1CON1bits.SPRE = 7;          // Secondary prescale 1:1
    SPI1CON1bits.PPRE = 2;          // Primary prescale 4:1
    SPI1CON2bits.SPIBEN = 0;          // No Enhanced mode
    //
    SPI1STATbits.SPIROV = 0;        // Clear overflow
    _SPI1IF = 0;                    // Clear interrupt flag
    _SPI1IE = 1;                    // Enable interrupt
    _SPI1IP = 1;                    // Interrupt priority
    SPI1STATbits.SISEL = 2;         // Interrupt when buffer 2/3 full
	SPI1STATbits.SPIEN = 1;
}

/* SPI2 Rx interrupt ISR */

void __attribute__((interrupt, auto_psv)) _SPI1Interrupt (void)
{
	char d;
	int i;


	d = SPI1BUF;			/* Get received data */
	_SPI1IF = 0;				/* Clear Rx interrupt flag */
	i = SPI1RxFifo.ct;				/* Number of bytes in the FIFO */
	if (i < BUFFER_SIZE) {		/* Skip if FIFO is full */
		SPI1RxFifo.ct = ++i;
		i = SPI1RxFifo.wi;
		SPI1RxFifo.buff[i++] = d;	/* Store data into the FIFO */
		SPI1RxFifo.wi = i % BUFFER_SIZE;	/* Next write ptr */
	}
}

int spi1_test (void)
{
	return SPI1RxFifo.ct;	/* Returns number of bytes in the Rx2 FIFO */
}

char spi1_getc (void)
{
	char d;
	int i;

	while (!SPI1RxFifo.ct) ;		/* Wait while Rx FIFO empty */

	i = SPI1RxFifo.ri;				/* Get a byte from Rx FIFO */
	d = SPI1RxFifo.buff[i++];
	SPI1RxFifo.ri = i % BUFFER_SIZE;
	__builtin_disable_interrupts();
	SPI1RxFifo.ct--;
	__builtin_enable_interrupts();

	return d;
}