//
// Vinculum VNC1L routines. Based on code from Matthias Kahnt, e-mail: az51@gmx.net
// Adapted for PIC24 by Anders Gustafsson. anders.gustafsson@pedago.fi
//
#include <xc.h>
#include <stdio.h>
#include <string.h>
#include "system.h"         // Defines config bits, FCY, VERSION etc
#include <libpic30.h>        // __delayXXX() functions macros defined here
#include "vnc1l.h"
#include "trigger.h"

int HILTasks(void);

int keyboardIs = -1;
int mouseIs = -1;

char command[16];
// ***********************************************************************
// VNC1L - Intialisierung SPI-Interface und VNC1L
// SPI Mode
// Returns 0b01 if nothing, 0b11 if keyboard, 0b101 if mouse 0b111 if both
// ***********************************************************************
int vnc_init(void) {
    char rc;
    char initstatus=0;
    char temp;
    int data;
    long count;
    char buffer[256];
    
    printf("\r\nHPKBD300-VNC1L Starting. %s %s", __DATE__, __TIME__);
    
    TRISBbits.TRISB4 = 0;   // Scope trigger output
    
    VNC1L_RESET_TRIS = 0;
    VNC1L_CS_TRIS = 0;
    VNC1L_CS_LAT = 0;       // VNC1L CS LOW
    VNC1L_RESET_LAT = 0;    // Hardware-Reset VNC1L		
                            //Pause
    count = 100;
    while(count--)
        HILTasks();         // One task is approx 12us
    VNC1L_RESET_LAT = 1;	// Start VNC1L
    //        
    // Pause, play nice and call HILTasks() when waiting
    //
    count = 300000;
    while(count--)
        HILTasks();
    //
    // Test code, read status register. if zero then there is no VNC1L
    //
    rc=vnc_rd_spi(1, &data);
    printf("\r\nFeeler: rc:%d %X\r\n", rc, data);
    if(data == 0) return 0;
    //
    //VNC1L found
    initstatus = 1;
    trigger(1);
    rc = vnc_response(buffer, sizeof(buffer),900);
    printf("\r\nResp1 %s ", buffer);
    trigger(2);
    rc = vnc_response(buffer, sizeof(buffer),100);
    printf("\r\nResp2 %s ", buffer);
    trigger(3);
    rc = vnc_response(buffer, sizeof(buffer),1);
    printf("\r\nResp3 %s ", buffer);

    rc = vnc_response(buffer, sizeof(buffer),1);
    printf("\r\nResp4 %s ", buffer);

    printf("\r\n\r\nWaiting");
    __delay_ms(5);
    printf("\r\nSend IPA");
    rc = vnc_wr_cmd("IPA");
    VncWaitForPrompt(buffer, sizeof(buffer));
    printf("\r\nSend ECS");
    rc = vnc_wr_cmd("ECS");
    VncWaitForPrompt(buffer, sizeof(buffer));
    rc = vnc_wr_cmd("FWV");
    printf("\r\n\r\nSend FWV complete");
    printf("\r\nrc: %X", rc);
    rc = vnc_response(buffer, sizeof(buffer),100);
    rc = vnc_response(buffer, sizeof(buffer),100);
    VncWaitForPrompt(buffer, sizeof(buffer));

    rc = vnc_wr_cmd("QP2");
    printf("\r\n QP2 rc: %X", rc);
    if(!rc)
    {
        rc = vnc_response(buffer, sizeof(buffer),100);
        //
        // check for a HID device
        //
        temp = buffer[2] - '0';
        if(temp & 0x08) printf("\nHID Device detected");
    }
    else return 0;
    VncWaitForPrompt(buffer, sizeof(buffer));
    //
    // Query first device
    //
    rc = vnc_wr_cmd("QD 0");

    printf("\r\n QD 0 rc: %X\r\n", rc);
    if(!rc)
    {
        rc = vnc_response(buffer, sizeof(buffer),100);
        printf("Response: %s \r\n", buffer);
        //
        // check what it is
        //
        temp = buffer[34] - '0';
        printf("\r\nHID Device class %X",temp);
        temp = buffer[54] - '0';
        if(temp>0) printf("\r\nHID Device protocol %X - %s ",temp,temp==1?"keyboard":"mouse");
        if(temp==1) keyboardIs = 0;
        if(temp==2) mouseIs = 0;
    }
    VncWaitForPrompt(buffer, sizeof(buffer));
    //
    // Query second device
    //
    rc = vnc_wr_cmd("QD 1");

    printf("\r\n QD 1 rc: %X\r\n", rc);
    if(!rc)
    {
        rc = vnc_response(buffer, sizeof(buffer),100);
        printf("Response: %s \r\n", buffer);
        //
        // check what it is
        //
        temp = buffer[34] - '0';
        printf("\nHID Device class %X",temp);
        temp = buffer[54] - '0';
        if(temp>0) printf("\r\nHID Device protocol %X - %s ",temp,temp==1?"keyboard":"mouse");
        if(temp==1) keyboardIs = 1;
        if(temp==2) mouseIs = 1;
    }
    VncWaitForPrompt(buffer, sizeof(buffer));
    //
    // no devices?
    if(keyboardIs == -1 && mouseIs == -1)
        return 1;

    sprintf(command, "SC 0");
    rc = vnc_wr_cmd(command);
    //printf("\r\n %s rc: %X ", command, rc);
    VncWaitForPrompt(buffer, sizeof(buffer));
    
    sprintf(command, "SC %d", keyboardIs);

    rc = vnc_wr_cmd(command);
    printf("\r\n %s rc: %X ", command, rc);
    VncWaitForPrompt(buffer, sizeof(buffer));

    // Device Send Setup Data - Set Idle
    //vnc.status = vnc_wr_cmd_UART("SSU $210A000000000000");
    // no interrupt transfer unless there is a change in the keycode
    // $210A007D00000000 => interrupt transfer every 500ms (500ms/4ms = 0x7D)
    //delay 120ms
    count = 100000;
    while(count--)
        HILTasks();
    rc = vnc_wr_cmd("SSU $210A007D00000000");

    printf("\r\nSSU $210A000000000000 rc: %X\r\n", rc);
    VncWaitForPrompt(buffer, sizeof(buffer));
        rc = vnc_wr_cmd("SSU $210A007D00000000");

    printf("\r\nSSU $210A000000000000 rc: %X\r\n", rc);
    VncWaitForPrompt(buffer, sizeof(buffer));
    //
    sprintf(command, "SC %d", mouseIs);

    rc = vnc_wr_cmd(command);
    printf("\r\n %s rc: %X ", command, rc);
    VncWaitForPrompt(buffer, sizeof(buffer));

    // Device Send Setup Data - Set Idle
    //vnc.status = vnc_wr_cmd_UART("SSU $210A000000000000");
    // no interrupt transfer unless there is a change in the keycode
    // $210A007D00000000 => interrupt transfer every 500ms (500ms/4ms = 0x7D)
        //delay 120ms
    count = 100000;
    while(count--)
        HILTasks();
    rc = vnc_wr_cmd("SSU $210A000000000000");

    printf("\r\nSSU $210A000000000000 rc: %X\r\n", rc);
    VncWaitForPrompt(buffer, sizeof(buffer));
        rc = vnc_wr_cmd("SSU $210A000000000000");

    printf("\r\nSSU $210A000000000000 rc: %X\r\n", rc);
    VncWaitForPrompt(buffer, sizeof(buffer));

    printf("\r\nStart polling");
    if(keyboardIs != -1) initstatus = initstatus | VNC1L_KEYBOARD_PRESENT;
    if(mouseIs != -1) initstatus = initstatus | VNC1L_MOUSE_PRESENT;
    return initstatus;
}
// ***********************************************************************
// VNC1L - Senden eines Zeichens
// bergabe: wr_byte   : Auszugebendes Zeichen
// Rckgabe: status    : 0=Accept (ok) 1=Byte nicht vom VNC1L angenommen
// ***********************************************************************
unsigned char vnc_wr_byte(unsigned char wr_byte) {
unsigned char status;
unsigned int cnt;
int data;
  cnt = TIMEOUT_READ_WRITE;                 // How long do we wait?
  do {
    // Read VNC1L status byte
    if (vnc_rd_spi(1, &data)) return 1;		// read error
	// Statusbyte liegt jetzt im Buffer
	status = data & 0x01;                   // Test VNC1L RECEIVE-BUFFER full (RXF# Bit0=1)
    //printf("\r\n %X %X ", status, data);
    cnt--;
	if (status) __delay_us(150);			// Pause, if RECEIVE-BUFFER full (RXF#)
  } while (status && cnt);
  if (status || (!cnt)) return 1;           // Error, nothing sent
  // VNC1L ist jetzt empfangsbereit
  if (vnc_wr_spi(0,wr_byte)) return 1;        // Error, nothing sent
  return 0;                                 // Byte wurde gesendetwas sent
}
 // Read from SPI
 //fn  (0=data, 1=Status)
 //data, pointer to returned data
 //returns 0 if new data, nonzero otherwise
unsigned char vnc_rd_spi(unsigned char fn, int *data)
{
    int d;
    int rc;
    // So data is 16 bits as:
    //
    VNC1L_CS_LAT = 1;               // Assert CS
    d = 0b1101111111110000;
    if(fn) d = d | 0b0010000000000000;    // Address bit
    SPI1BUF = d;
    while(SPI1STATbits.SPITBF);  //While full wait
    while(!SPI1STATbits.SPIRBF);  //While full wait
    rc = SPI1BUF;
    *data = rc>>5;
    //printf("\r\n %X \r\n", *data);
    // Check status bit;
    //__delay_us(1);
    VNC1L_CS_LAT = 0;               // Deassert CS
    return (rc & 0b0000000000010000);
}

//
// write to SPI
// fn  (0=data, 1=Status)
// data, pointer to returned data
// returns 0 if data accepted, nonzero otherwise
unsigned char vnc_wr_spi(unsigned char fn, int data)
{
    int d;
    int rc;

    VNC1L_CS_LAT = 1;               // Assert CS
    // So data is 16 bits as:
    //
    //printf(" wr_spi %X %c ",data, data);
    d = 0b1000000000000000;
    if(fn) d = d | 0b0010000000000000;    // Address bit
    SPI1BUF = d | (data <<5);
    //printf("%d",SPI1STATbits.SPITBF);
    while(SPI1STATbits.SPITBF);  //While full wait
    //printf("%d",SPI1STATbits.SPITBF);
    while(!SPI1STATbits.SPIRBF);  //While full wait
    //printf("c");
    rc = SPI1BUF;
    // Check status bit
    //__delay_us(1);
    VNC1L_CS_LAT = 0;               // Deassert CS
    //printf(" rc %X ",rc);
    return (rc & 0b0000000000010000);
}

// ***********************************************************************
// VNC1L - Send a command
// Parameter: wr_cmd : command (address)
// Return: status : 0 = OK, 1 = command not accepted by VNC1L
// ***********************************************************************
unsigned char vnc_wr_cmd(char *wr_cmd)
{
unsigned char offset; // vnc_offset, length of command
 for(offset = strlen(wr_cmd); offset; offset--)
 {
 if (vnc_wr_byte(*wr_cmd)) return 1; // error (overflow), command was not accepted
 wr_cmd++;
 }
 // send carrage return (a command to monitor ends with 0x0D)
 if (vnc_wr_byte(0x0D)) return 1; // error (overflow), command was not accepted
 return 0;
}
//
// Read a response, terminated by 0x0d
//
unsigned char vnc_response(char *buffer, int buflen, int timeout)
{
    unsigned long count;
    int bufptr = 0;
    int data;
    char rc;
    bool more = true;
    
    count = timeout * 1000;
    while(count-- && more && bufptr < buflen)
    {
        HILTasks();
        rc = vnc_rd_spi(0, &data);
        if(rc==0)
        {
            buffer[bufptr++] = data;
            //printf("%c", data);
            if (data == 0x0d)
            {
                buffer[bufptr] = 0;
                //printf("\r\n%s\r\n", buffer);
                more = false;
                rc = 0;
                return 0;
            }
        }
        //__delay_ms(10);
    }
    //printf("\r\nresponse timeout");
    rc = 1;
    return 1;
}
//
// Wait for VNC1L prompt. Returns 0 on success, 1 on error (timeout))
unsigned char VncWaitForPrompt(char *buffer, int buflen)
{
    int count;
    
    count = 300;
    while(count--)
    {
        vnc_response(buffer,buflen,1);
        //printf("\r\nWaitForPrompt, buffer: %s", buffer);
        if(strstr(buffer,"D:\\>")) return 0;
    }
    printf("\r\nWaitForPromt timeout");
    return 1;
}