//
// Vinculum VNC2 routines. 
// Adapted for PIC24 by Anders Gustafsson. anders.gustafsson@pedago.fi
//
#include <xc.h>
#include <stdio.h>
#include <string.h>
#include "system.h"         // Defines config bits, FCY, VERSION etc
#include <libpic30.h>        // __delayXXX() functions macros defined here
#include "vnc2.h"
#include "ps2_mouse.h"
#include "hil_kbd.h"
#include "trigger.h"
#include "spi_slave.h"

char packet[30];
int usbptr = 0;
int usbcntr = 0;
int tick = 0;

// VNC2 does the bulk of USB work, sending us nice packets as you press
// the kays.
//
// Packet format:
// 0x13
// length, 4 or 8
// byte1 MS nibble
// byte1 LS nibble
// ..
// 0x14
int USBTasks(void)
{
    char ch;
    //int i;
    int len;
    
    if(spi1_test() == 0) return 0;
    //
    ch = spi1_getc();
    //printf(" %02X",ch);
    //
    // Assemble packet
    if(ch == 0x13)
    {
        usbptr = 0;
        usbcntr = 0;
        return 0;
    }
    if(ch == 0x14)
    {
//        printf("\r\n%d:",tick++);
//        for(i = 0; i<10;i++)
//            printf("%02X ", (packet[i] & 0xff));
//        printf("\r\n");
        usbptr = 0;
        usbcntr = 0;
        if(packet[0] == 8) USBKeyboardTasks(packet);
        if(packet[0] == 4) USBMouseTasks(packet);
        return 0;
    }

    if(usbcntr==0)
    {
        len = ch;
        packet[usbptr++] = ch;
        ++usbcntr;
        return 0;
    }
    ++usbcntr;
    if((usbcntr % 2)==0)
    {
        packet[usbptr] = (ch << 4);
    }
    else
    {
        packet[usbptr] = packet[usbptr] | ch;
        ++usbptr;
        //usbptr = usbptr & 0x0f;
    }
    return 0;
}
int USBKeyboardTasks(char *packet)
{
// packrt[0] length
// packet[1] status
// packet[2] 
// packet[3] scancode
//    int rc;
//    int count;
char stat;
char code;
//    char buffer[256];
char tempkey;
//    char command[16];
//    
    
    stat = packet[1]; // Keyboard status, ie shift and stuff
    code = packet[3]; // Keyboard scancode
    //printf("\r\nkbd %02X %02X",code,Lastcode);
    //
    // Emulate press and release
    //
    // Press
    if(code > 0 && Lastcode == 0)
    {
        printf("\r\nU:  %02X", code);
        Lastcode = code;
        uint8_t index2;
        for(index2 = 0; index2 < sizeof(keyTranslationTable)/sizeof(HID_KEY_TRANSLATION_TABLE_ENTRY); index2++)
        {
            if(keyTranslationTable[index2].key == code)
            {
                if(Code2)
                {
                    tempkey = keyTranslationTable[index2].modified;                   
                }
                else
                {
                    tempkey = keyTranslationTable[index2].unmodified;                   
                }
                //printf(" send: %2X ", tempkey);
                HILPutTX(tempkey);  // Put in send buffer FIFO
                LastKey = tempkey;
            }
        }
    } 
    // Release, send code + 1
    else if(Lastcode > 0 && code == 0)
    {
        printf("\r\nUR: %02X ", Lastcode);
        Lastcode = code;
        HILPutTX(LastKey+1);
    }
    
    // Has status changed? Ie any of Shift/Ctrl/Alt pressed?
    // If so, convert to scancodes
    //
    if(stat != Laststat)
    {
        printf(" Status: %X -> %X", Laststat, stat);
        
        if((stat & USB_LEFT_CTRL) != (Laststat & USB_LEFT_CTRL))
        {
           printf("\r\nL ctrl changed to: %d ", (stat & USB_LEFT_CTRL));
           if (stat & USB_LEFT_CTRL)
           {
               printf("press");
               HILPutTX(0x0C);
           }
           else 
           {
               printf("release");
               HILPutTX(0x0D);
           }
        }
        if((stat & USB_LEFT_SHIFT) != (Laststat & USB_LEFT_SHIFT))
        {
           printf("\r\nL shift changed to: %d ", (stat & USB_LEFT_SHIFT));
           if (stat & USB_LEFT_SHIFT)
           {
               printf("press");
               HILPutTX(0x0A);
           }
           else 
           {
               printf("release");
               HILPutTX(0x0B);
           }
        }
        if((stat & USB_LEFT_ALT) != (Laststat & USB_LEFT_ALT))
        {
           printf("\r\nL alt changed to: %d ", (stat & USB_LEFT_ALT));
           if (stat & USB_LEFT_ALT)
           {
               printf("press");
               HILPutTX(0x06);
           }
           else 
           {
               printf("release");
               HILPutTX(0x07);
           }
        }
        //
        // Our own shift to access extra keys
        //
        if((stat & USB_RIGHT_CTRL) != (Laststat & USB_RIGHT_CTRL))
        {
            if(stat & USB_RIGHT_CTRL)
            {
                Code2 = 1;
                printf(" code2 on ");
            }
            else
            {
                Code2 = 0;
                printf(" code2 off ");
            }
        }
        Laststat = stat;
    } 
    return 1;
}

int USBMouseTasks(char *packet)
{
// packet[0] length
// packet[1] buttons
// packet[2] x
// packet[3] y
    int count;
    char button;
    char x;
    char y;

    count = 4;
    button=0;
    x=0;
    y=0;
    //
    // Parse 4 bytes of mouse data

    button = packet[1];
    x = packet[2];
    y = packet[3];
    //printf("%0X ", data);
    //if (data >= '0') printf("%c ", data); else printf(". ");
    //if (data == 0x0d) printf("\r\n");
    //if(x != 0) printf("\r\n%X %X %X", button, x, y);
    if(x!=0 || y!=0)        // Coordinate change/move?
    {
        mousex = x;
        mousey = -y;
        Mouse = 1;
    }
    if(button != Lastbutton)    // Button Press?
    {
        if(button & USB_HID_MOUSE_LEFT)
            {
                MouseStatus.left = 1; // Left
            }
            else MouseStatus.left = 0;
            if(button & USB_HID_MOUSE_RIGHT)
            {
                MouseStatus.right = 1; // right
            }
            else MouseStatus.right = 0;
        Mouse = 1;
    }
    Lastbutton = button;
    return 1;
}
